package impl

import (
	"fmt"
	"math"
	"math/rand"
	"time"
	"unicode"

	"gitlab.okta-solutions.com/mashroom/backend/common/generic/env"
	invitation "gitlab.okta-solutions.com/mashroom/backend/invitation/api"
)

const (
	EmailOTPLength = 6
	SmsOTPLength   = 5
)

var (
	TestOtpPrefix = env.ResolveString("TEST_OTP_PREFIX", "")
)

func init() {
	rand.Seed(time.Now().UTC().UnixNano())
}

// Generates random hex string of required length.
func generateEmailOTP() string {
	r := rand.Intn(int(math.Pow10(EmailOTPLength)))
	format := fmt.Sprintf("%%0%dd", EmailOTPLength)
	return fmt.Sprintf(format, r)
}

// Generates random decimal number of required length.
func generateSmsOTP() string {
	r := rand.Intn(int(math.Pow10(SmsOTPLength)))
	format := fmt.Sprintf("%%0%dd", SmsOTPLength)
	return fmt.Sprintf(format, r)
}

func generateOTP(t invitation.ContactValidationType) string {
	switch t {
	case invitation.ContactValidationType_EMAIL_VALIDATION:
		return generateEmailOTP()
	case invitation.ContactValidationType_PHONE_VALIDATION:
		return generateSmsOTP()
	default:
		return ""
	}
}

func generateTestOTP(contact string) (string, bool) {
	if TestOtpPrefix == "" || contact == "" {
		return "", false
	}

	result := ""
	for _, v := range contact {
		if unicode.IsDigit(v) {
			result = string(v) + result
		}
	}

	if result == "" {
		return "", false
	}

	return TestOtpPrefix + result, true
}
