package impl

import (
	"regexp"
	"testing"

	invitation "gitlab.okta-solutions.com/mashroom/backend/invitation/api"
)

func TestGenerateOTP(t *testing.T) {
	otp := generateOTP(invitation.ContactValidationType_UNKNOWN_VALIDATION)
	if otp != "" {
		t.Errorf("Wrong OTP for unknown type : '%s'.", otp)
	}

	// email otp
	otp = generateOTP(invitation.ContactValidationType_EMAIL_VALIDATION)
	if len(otp) != EmailOTPLength {
		t.Errorf("Wrong email OTP length: '%s'.", otp)
	}
	ok, err := regexp.MatchString("^[0-9a-f]+$", otp)
	if err != nil || !ok {
		t.Errorf("Email OTP should be hex digits only: '%s'.", otp)
	}

	// sms otp
	otp = generateOTP(invitation.ContactValidationType_PHONE_VALIDATION)
	if len(otp) != SmsOTPLength {
		t.Errorf("Wrong phone OTP length: '%s'.", otp)
	}
	ok, err = regexp.MatchString("^[0-9]+$", otp)
	if err != nil || !ok {
		t.Errorf("Phone OTP should be digits only: '%s'.", otp)
	}
}
