package impl

import (
	"context"
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"

	"github.com/globalsign/mgo/bson"
	invitation "gitlab.okta-solutions.com/mashroom/backend/invitation/api"
)

type ContactValidationModel struct {
	Id        primitive.ObjectID               `bson:"_id,omitempty"`
	AccountId string                           `bson:"account_id"`
	Key       string                           `bson:"key"`
	Otp       string                           `bson:"otp"`
	Body      string                           `bson:"body"`
	Type      invitation.ContactValidationType `bson:"type"`
	CreatedAt time.Time                        `bson:"created_at"`
	ExpiresAt time.Time                        `bson:"expires_at"`
}

func (repo *Repo) CreateContactValidation(ctx context.Context, model *ContactValidationModel) (primitive.ObjectID, error) {
	_, err := repo.contactValidations.InsertOne(ctx, model)
	return model.Id, err
}

func (repo *Repo) RemoveContactValidations(ctx context.Context, t invitation.ContactValidationType, accountId string, key string) (int, error) {
	selector := bson.M{"type": t, "account_id": accountId, "key": key}
	info, err := repo.contactValidations.DeleteMany(ctx, selector)
	if err != nil {
		return 0, err
	}

	return int(info.DeletedCount), err
}

func (repo *Repo) GetContactValidation(ctx context.Context, t invitation.ContactValidationType, accountId string, key string) ([]*ContactValidationModel, error) {
	var models []*ContactValidationModel

	selector := bson.M{"type": t, "account_id": accountId, "key": key}

	cur, err := repo.contactValidations.Find(ctx, selector)
	if err != nil {
		return nil, err
	}

	err = cur.All(ctx, &models)
	if err != nil {
		return nil, err
	}

	return models, nil
}
