package impl

import (
	"net"

	"github.com/zeebe-io/zeebe/clients/go/pkg/zbc"
	"gitlab.okta-solutions.com/mashroom/backend/common/api/health"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/env"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/error/mongoerr"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/grpc"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/log"
	invitation "gitlab.okta-solutions.com/mashroom/backend/invitation/api"
	"gitlab.okta-solutions.com/mashroom/backend/invitation/version"
	notification "gitlab.okta-solutions.com/mashroom/backend/notification/api"
)

var (
	NotificationServiceUrl = env.ResolveString("NOTIFICATION_SERVICE_URL", "mashroom-notification:10000")
	SiteUrl                = env.ResolveString("SITE_URL", "staging.env.mashroom.com")
)

type Server interface {
	invitation.InvitationServiceServer
	Serve(addr string)
	Background()
}

type serverImpl struct {
	Server
	repo         *Repo
	notification notification.NotificationServiceClient
	bpms         zbc.Client
}

func (server *serverImpl) Background() {
	// background processes
}

func (server *serverImpl) Serve(addr string) {
	if listener, err := net.Listen("tcp", addr); err != nil {
		panic(err)
	} else {
		grpcServer, err := grpc.NewServer("invitation", grpc.ServerOptionErrorTranslator(
			mongoerr.Translate,
		))
		if err != nil {
			panic(err)
		}

		invitation.RegisterInvitationServiceServer(grpcServer, server)

		healthServer := version.NewHealthServer()
		health.RegisterHealthServiceServer(grpcServer, healthServer)

		log.Info("invitation started")
		if err := grpcServer.Serve(listener); err != nil {
			log.Error(err)
		}
	}
}

func NewServer(repo *Repo) Server {
	server := &serverImpl{repo: repo}

	server.notification = notification.NewNotificationServiceClient(grpc.Dial(NotificationServiceUrl, "Notification"))

	bpms, err := server.ServeBpms()
	if err != nil {
		panic(err)
	}
	server.bpms = bpms

	go server.Background()
	return server
}
